@@ -25,11 +25,12 @@
 import logging, re
 from collections import OrderedDict
 
+from django.core.exceptions import FieldError
 from django.db import transaction
-from django.db.models import F, Max
+from django.db.models import F, Max, Q
 from django.http import Http404
 from django.shortcuts import get_object_or_404
-from rest_framework import generics, response as http
+from rest_framework import generics, response as http, status
 from rest_framework.mixins import CreateModelMixin
 from rest_framework.filters import OrderingFilter, SearchFilter
 from rest_framework.pagination import PageNumberPagination
@@ -55,25 +56,30 @@ class MatrixCreateAPIView(generics.ListCreateAPIView):
 
     .. code-block:: http
 
-        GET /api/matrix/ HTTP/1.1
+        GET /api/energy-utility/reporting/sustainability/matrix HTTP/1.1
 
     responds
 
     .. code-block:: json
 
         {
-           "slug": "all",
-           "title": "All accounts against all questions",
-           "metric": {
-               "slug": "all-questions",
-               "title": "All questions",
-               "predicates": []
-           },
-           "cohorts": [{
-               "slug": "all-accounts",
-               "title": "All accounts",
-               "predicates": []
-           }]
+          "count": 1,
+          "results": [
+            {
+              "slug": "all",
+              "title": "All accounts against all questions",
+              "metric": {
+                "slug": "all-questions",
+                "title": "All questions",
+                "predicates": []
+              },
+              "cohorts": [{
+                 "slug": "all-accounts",
+                 "title": "All accounts",
+                 "predicates": []
+              }]
+            }
+          ]
         }
     """
     serializer_class = MatrixSerializer
@@ -89,7 +95,7 @@ class MatrixCreateAPIView(generics.ListCreateAPIView):
 
         .. code-block:: http
 
-            POST /api/matrix/ HTTP/1.1
+            POST /api/energy-utility/reporting/sustainability/matrix HTTP/1.1
 
         .. code-block:: json
 
@@ -147,14 +153,14 @@ class MatrixDetailAPIView(MatrixMixin, generics.RetrieveUpdateDestroyAPIView):
 
     .. code-block:: json
 
-        [{
+        {
            "slug": "languages",
            "title": "All cohorts for all questions",
            "values": {
                "portfolio-a": 0.1,
                "portfolio-b": 0.5
            }
-        }]
+        }
     """
 
     serializer_class = MatrixSerializer
@@ -185,14 +191,11 @@ class MatrixDetailAPIView(MatrixMixin, generics.RetrieveUpdateDestroyAPIView):
 
         .. code-block:: json
 
-            [{
+            {
               "slug": "languages",
               "title": "All cohorts for all questions",
-              "values":{
-                "portfolio-a": 0.1,
-                "portfolio-b": 0.5
-             }
-            }]
+              "cohorts": []
+            }
         """
         #pylint:disable=useless-super-delegation
         return super(MatrixDetailAPIView, self).put(
@@ -253,13 +256,14 @@ class MatrixDetailAPIView(MatrixMixin, generics.RetrieveUpdateDestroyAPIView):
 
     @property
     def matrix(self):
+        #pylint:disable=attribute-defined-outside-init
         if not hasattr(self, '_matrix'):
             self._matrix = Matrix.objects.filter(
                 slug=self.kwargs.get(self.matrix_url_kwarg)).first()
         return self._matrix
 
     def get_accounts(self):
-        #pylint:disable=unused-argument,no-self-use
+        #pylint:disable=unused-argument
         return get_account_model().objects.all()
 
     def get_likely_metric(self, cohort_slug):
@@ -343,11 +347,11 @@ class MatrixDetailAPIView(MatrixMixin, generics.RetrieveUpdateDestroyAPIView):
         return http.Response(result)
 
 
-class EditableFilterQuerysetMixin(object):
+class EditableFilterQuerysetMixin(AccountMixin):
 
-    @staticmethod
-    def get_queryset():
-        return EditableFilter.objects.all().order_by('slug')
+    def get_queryset(self):
+        return EditableFilter.objects.filter(
+            Q(account__isnull=True) | Q(account=self.account)).order_by('slug')
 
 
 class EditableFilterListAPIView(EditableFilterQuerysetMixin,
@@ -402,6 +406,7 @@ class EditableFilterDetailAPIView(AccountMixin,
 
     @property
     def editable_filter(self):
+        #pylint:disable=attribute-defined-outside-init
         if not hasattr(self, '_editable_filter'):
             self._editable_filter = get_object_or_404(
                 EditableFilter.objects.all(),
@@ -420,7 +425,7 @@ class EditableFilterDetailAPIView(AccountMixin,
 class AccountsFilterDetailAPIView(CreateModelMixin,
                                   EditableFilterDetailAPIView):
     """
-    Retrieves a fitler
+    Retrieves a profiles fitler
 
     **Tags**: reporting
 
@@ -434,14 +439,13 @@ class AccountsFilterDetailAPIView(CreateModelMixin,
 
     .. code-block:: json
 
-        {
-            "count": 1,
-            "previous": null,
-            "next": null,
-            "slug": "boxes-and-enclosures",
-            "title": "Boxes & enclosures",
-            "predicates": [],
-            "results": [{
+         {
+           "slug": "suppliers",
+           "title": "Energy utility suppliers",
+           "extra": null,
+           "predicates": [],
+           "likely_metric": null,
+           "results": [{
                 "facility": "Main factory",
                 "fuel_type": "natural-gas",
                 "allocation": "Energy utility",
@@ -472,36 +476,30 @@ class AccountsFilterDetailAPIView(CreateModelMixin,
         .. code-block:: json
 
             {
-                "slug": "suppliers",
                 "title": "Energy utility suppliers",
-                "tags": "cohort, aggregate",
-                "predicates": [{
-                    "rank": 1,
-                    "operator": "contains",
-                    "operand": "Energy",
-                    "field": "extra",
-                    "selector": "keepmatching"
-                }],
-                "likely_metric": null
+                "predicates": []
             }
 
         responds
 
         .. code-block:: json
 
-            {
-                "slug": "suppliers",
-                "title": "Energy utility suppliers",
-                "tags": "cohort, aggregate",
-                "predicates": [{
-                    "rank": 1,
-                    "operator": "contains",
-                    "operand": "Energy",
-                    "field": "extra",
-                    "selector": "keepmatching"
-                }],
-                "likely_metric": null
-            }
+         {
+           "slug": "suppliers",
+           "title": "Energy utility suppliers",
+           "extra": null,
+           "predicates": [],
+           "likely_metric": null,
+           "results": [{
+                "facility": "Main factory",
+                "fuel_type": "natural-gas",
+                "allocation": "Energy utility",
+                "created_at": null,
+                "ends_at": null,
+                "amount": 100,
+                "unit": "mmbtu"
+            }]
+        }
         """
         #pylint:disable=useless-super-delegation
         return super(AccountsFilterDetailAPIView, self).put(
@@ -509,7 +507,7 @@ class AccountsFilterDetailAPIView(CreateModelMixin,
 
     def delete(self, request, *args, **kwargs):
         """
-        Deletes a fitler
+        Deletes a profiles fitler
 
         **Tags**: reporting
 
@@ -525,7 +523,7 @@ class AccountsFilterDetailAPIView(CreateModelMixin,
 
     def post(self, request, *args, **kwargs):
         """
-        Updates a fitler
+        Updates a profiles fitler
 
         **Tags**: reporting
 
@@ -553,13 +551,19 @@ class AccountsFilterDetailAPIView(CreateModelMixin,
         #pylint:disable=useless-super-delegation
         return self.create(request, *args, **kwargs)
 
-    def perform_create(self, serializer):
+    def create(self, request, *args, **kwargs):
+        serializer = self.get_serializer(data=request.data)
+        serializer.is_valid(raise_exception=True)
+
+        # Create the `Account` (if necessary) and add it to the filter.
         with transaction.atomic():
             account_slug = serializer.validated_data.get('slug')
             extra = serializer.validated_data.get('extra')
             if account_slug:
-                account = get_object_or_404(get_account_model().objects.all(),
-                    slug=account_slug)
+                account_queryset = get_account_model().objects.all()
+                account_lookup_field = settings.ACCOUNT_LOOKUP_FIELD
+                filter_args = {account_lookup_field: account_slug}
+                account = get_object_or_404(account_queryset, **filter_args)
             else:
                 account = get_account_model().objects.create(
                     full_name=serializer.validated_data.get('full_name'),
@@ -569,15 +573,21 @@ class AccountsFilterDetailAPIView(CreateModelMixin,
                 Max('rank')).get('rank__max')
             if not last_rank:
                 last_rank = 0
-            EditableFilterEnumeratedAccounts.objects.create(
+            enum_account = EditableFilterEnumeratedAccounts.objects.create(
                 account=account,
                 editable_filter=self.editable_filter,
                 rank=last_rank + 1)
+            account.rank = enum_account.rank
+
+        account_serializer = get_account_serializer()()
+        headers = self.get_success_headers(serializer.data)
+        return http.Response(account_serializer.to_representation(account),
+            status=status.HTTP_201_CREATED, headers=headers)
 
 
 class AccountsFilterEnumeratedAPIView(generics.RetrieveUpdateDestroyAPIView):
     """
-    Retrieves a single account inside a accounts filter
+    Retrieves a profile in an enumerated profiles filter
 
     **Tags**: reporting
 
@@ -591,31 +601,13 @@ class AccountsFilterEnumeratedAPIView(generics.RetrieveUpdateDestroyAPIView):
 
     .. code-block:: json
 
-        {
-            "count": 1,
-            "previous": null,
-            "next": null,
-            "title": "Energy utility suppliers",
-            "predicates": [{
-                "rank": 1,
-                "operator": "contains",
-                "operand": "Energy",
-                "field": "extra",
-                "selector": "keepmatching"
-            }],
-            "results": [{
-                "facility": "Main factory",
-                "fuel_type": "natural-gas",
-                "allocation": "Energy utility",
-                "created_at": null,
-                "ends_at": null,
-                "amount": 100,
-                "unit": "mmbtu"
-            }]
-        }
+         {
+           "slug": "main-factory",
+           "full_name": "Main factory"
+         }
     """
     lookup_field = 'rank'
-    serializer_class = EditableFilterSerializer
+    serializer_class = AccountsFilterAddSerializer
 
     def get_queryset(self):
         queryset = EditableFilterEnumeratedAccounts.objects.filter(
@@ -653,7 +645,7 @@ class AccountsFilterEnumeratedAPIView(generics.RetrieveUpdateDestroyAPIView):
 
     def put(self, request, *args, **kwargs):
         """
-        Updates a single account inside a accounts filter
+        Updates a profile in an enumerated profiles filter
 
         **Tags**: reporting
 
@@ -666,13 +658,7 @@ class AccountsFilterEnumeratedAPIView(generics.RetrieveUpdateDestroyAPIView):
         .. code-block:: json
 
             {
-                "facility": "Main factory",
-                "fuel_type": "natural-gas",
-                "allocation": "Energy utility",
-                "created_at": null,
-                "ends_at": null,
-                "amount": 100,
-                "unit": "mmbtu"
+                "full_name": "Main factory"
             }
 
         responds
@@ -680,21 +666,17 @@ class AccountsFilterEnumeratedAPIView(generics.RetrieveUpdateDestroyAPIView):
         .. code-block:: json
 
             {
-                "facility": "Main factory",
-                "fuel_type": "natural-gas",
-                "allocation": "Energy utility",
-                "created_at": null,
-                "ends_at": null,
-                "amount": 100,
-                "unit": "mmbtu"
+                "slug": "main-factory",
+                "full_name": "Main factory"
             }
         """
+        #pylint:disable=useless-parent-delegation
         return super(AccountsFilterEnumeratedAPIView, self).put(
             request, *args, **kwargs)
 
     def delete(self, request, *args, **kwargs):
         """
-        Deletes a single account inside a accounts filter
+        Deletes a profile in an enumerated profiles filter
 
         **Tags**: reporting
 
@@ -704,13 +686,14 @@ class AccountsFilterEnumeratedAPIView(generics.RetrieveUpdateDestroyAPIView):
 
              DELETE /api/energy-utility/filters/accounts/suppliers/1 HTTP/1.1
         """
+        #pylint:disable=useless-parent-delegation
         return super(AccountsFilterEnumeratedAPIView, self).delete(
             request, *args, **kwargs)
 
 
 class QuestionsFilterDetailAPIView(EditableFilterDetailAPIView):
     """
-    Retrieves a fitler
+    Retrieves a questions fitler
 
     **Tags**: reporting
 
@@ -718,30 +701,28 @@ class QuestionsFilterDetailAPIView(EditableFilterDetailAPIView):
 
     .. code-block:: http
 
-         GET /api/energy-utility/filters/questions/suppliers HTTP/1.1
+         GET /api/energy-utility/filters/questions/governance HTTP/1.1
 
     responds
 
     .. code-block:: json
 
         {
-            "slug": "suppliers",
-            "title": "Energy utility suppliers",
-            "tags": "cohort, aggregate",
+            "slug": "governance",
+            "title": "Governance questions",
             "predicates": [{
                 "rank": 1,
                 "operator": "contains",
                 "operand": "Energy",
                 "field": "extra",
                 "selector": "keepmatching"
-            }],
-            "likely_metric": null
+            }]
         }
     """
 
     def put(self, request, *args, **kwargs):
         """
-        Updates a fitler
+        Updates a questions fitler
 
         **Tags**: reporting
 
@@ -749,22 +730,20 @@ class QuestionsFilterDetailAPIView(EditableFilterDetailAPIView):
 
         .. code-block:: http
 
-             PUT /api/energy-utility/filters/questions/suppliers HTTP/1.1
+             PUT /api/energy-utility/filters/questions/governance HTTP/1.1
 
         .. code-block:: json
 
             {
-                "slug": "suppliers",
-                "title": "Energy utility suppliers",
-                "tags": "cohort, aggregate",
+                "slug": "governance",
+                "title": "Governance questions",
                 "predicates": [{
                     "rank": 1,
                     "operator": "contains",
                     "operand": "Energy",
                     "field": "extra",
                     "selector": "keepmatching"
-                }],
-                "likely_metric": null
+                }]
             }
 
         responds
@@ -772,17 +751,15 @@ class QuestionsFilterDetailAPIView(EditableFilterDetailAPIView):
         .. code-block:: json
 
             {
-                "slug": "suppliers",
-                "title": "Energy utility suppliers",
-                "tags": "cohort, aggregate",
+                "slug": "governance",
+                "title": "Governance questions",
                 "predicates": [{
                     "rank": 1,
                     "operator": "contains",
                     "operand": "Energy",
                     "field": "extra",
                     "selector": "keepmatching"
-                }],
-                "likely_metric": null
+                }]
             }
         """
         #pylint:disable=useless-super-delegation
@@ -791,7 +768,7 @@ class QuestionsFilterDetailAPIView(EditableFilterDetailAPIView):
 
     def delete(self, request, *args, **kwargs):
         """
-        Deletes a fitler
+        Deletes a questions fitler
 
         **Tags**: reporting
 
@@ -799,7 +776,7 @@ class QuestionsFilterDetailAPIView(EditableFilterDetailAPIView):
 
         .. code-block:: http
 
-             DELETE /api/energy-utility/filters/questions/suppliers HTTP/1.1
+             DELETE /api/energy-utility/filters/questions/governance HTTP/1.1
         """
         #pylint:disable=useless-super-delegation
         return super(QuestionsFilterDetailAPIView, self).delete(
@@ -828,7 +805,7 @@ class EditableFilterObjectsAPIView(generics.ListCreateAPIView):
     """
     Base class to filter accounts and questions
     """
-    pagination_class = EditableFilterPagination
+#    pagination_class = EditableFilterPagination
     serializer_class = None # override in subclasses
     lookup_field = 'slug'
     lookup_url_kwarg = 'editable_filter'
@@ -836,13 +813,6 @@ class EditableFilterObjectsAPIView(generics.ListCreateAPIView):
     def get_queryset(self):
         return self.get_serializer_class().Meta.model.objects.all()
 
-    def get(self, request, *args, **kwargs): #pylint: disable=unused-argument
-        self.editable_filter = generics.get_object_or_404(
-            EditableFilter.objects.all(),
-            slug=self.kwargs[self.lookup_url_kwarg])
-        return super(EditableFilterObjectsAPIView, self).get(
-            request, *args, **kwargs)
-
     def post(self, request, *args, **kwargs):
         """
         Creates a fitler
@@ -900,7 +870,7 @@ class EditableFilterObjectsAPIView(generics.ListCreateAPIView):
 
 class AccountsFilterListAPIView(EditableFilterObjectsAPIView):
     """
-    Filtered list of ``EditableFilter``.
+    Lists profiles filters
 
     **Tags**: reporting
 
@@ -926,12 +896,43 @@ class AccountsFilterListAPIView(EditableFilterObjectsAPIView):
            }]
         }
     """
-    serializer_class = get_account_serializer()
+    serializer_class = EditableFilterSerializer
+
+    def post(self, request, *args, **kwargs):
+        """
+        Creates a profiles fitler
+
+        **Tags**: reporting
+
+        **Examples**
+
+        .. code-block:: http
+
+             POST /api/energy-utility/filters/accounts HTTP/1.1
+
+        .. code-block:: json
+
+            {
+            }
+
+        responds
+
+        .. code-block:: json
+
+            {
+               "slug": "supplier-1",
+               "printable_name": "Supplier 1",
+               "extra": null
+            }
+        """
+        #pylint:disable=useless-super-delegation
+        return super(AccountsFilterListAPIView, self).post(
+            request, *args, **kwargs)
 
 
 class QuestionsFilterListAPIView(EditableFilterObjectsAPIView):
     """
-    Filtered list of ``Question``.
+    Lists questions filters
 
     **Tags**: reporting
 
@@ -952,35 +953,43 @@ class QuestionsFilterListAPIView(EditableFilterObjectsAPIView):
             "results": [{
               "path": "/construction/product-design",
               "title": "Product Design",
-              "default_unit": {
-                  "slug": "assessment",
-                  "title": "assessments",
-                  "system": "enum",
-                  "choices": [
-                    {
-                      "rank": 1,
-                      "text": "mostly-yes",
-                       "descr": "Mostly yes"
-                    },
-                    {
-                      "rank": 2,
-                      "text": "yes",
-                      "descr": "Yes"
-                    },
-                    {
-                      "rank": 3,
-                      "text": "no",
-                      "descr": "No"
-                    },
-                    {
-                      "rank": 4,
-                      "text": "mostly-no",
-                      "descr": "Mostly no"
-                    }
-                  ]
-              },
+              "default_unit": "assessment",
               "ui_hint": "radio"
             }]
         }
     """
     serializer_class = get_question_serializer()
+
+    def post(self, request, *args, **kwargs):
+        """
+        Creates a questions fitler
+
+        **Tags**: reporting
+
+        **Examples**
+
+        .. code-block:: http
+
+             POST /api/energy-utility/filters/questions HTTP/1.1
+
+        .. code-block:: json
+
+            {
+                "title": "Construction",
+                "default_unit": "assessment",
+                "path": "/construction/product-design"
+            }
+
+        responds
+
+        .. code-block:: json
+
+            {
+                "title": "Construction",
+                "default_unit": "assessment",
+                "path": "/construction/product-design"
+            }
+        """
+        #pylint:disable=useless-super-delegation
+        return super(QuestionsFilterListAPIView, self).post(
+            request, *args, **kwargs)
