@@ -24,16 +24,19 @@
 
 import logging
 
+from dateutil.relativedelta import relativedelta
 from django.db import transaction
 from django.db.models import Q
 from rest_framework import generics, status
+from rest_framework.exceptions import ValidationError
 from rest_framework.response import Response as HttpResponse
 
 from .. import settings, signals
-from ..compat import six
+from ..compat import six, gettext_lazy as _
+from ..docs import OpenAPIResponse, swagger_auto_schema
 from ..mixins import AccountMixin
 from ..models import PortfolioDoubleOptIn
-from .serializers import (PortfolioOptInSerializer,
+from .serializers import (NoModelSerializer, PortfolioOptInSerializer,
     PortfolioGrantCreateSerializer, PortfolioRequestCreateSerializer)
 from ..filters import (DateRangeFilter, DoubleOptInStateFilter, OrderingFilter,
      SearchFilter)
@@ -158,6 +161,14 @@ class PortfoliosGrantsAPIView(SmartPortfolioListMixin,
             Q(account=self.account) &
             Q(state=PortfolioDoubleOptIn.OPTIN_GRANT_INITIATED))
 
+    def get_serializer_class(self):
+        if self.request.method.lower() == 'post':
+            return PortfolioGrantCreateSerializer
+        return super(PortfoliosGrantsAPIView, self).get_serializer_class()
+
+    @swagger_auto_schema(responses={
+        201: OpenAPIResponse("Create successful", PortfolioOptInSerializer,
+        many=True)})
     def post(self, request, *args, **kwargs):
         """
         Initiates a grant
@@ -175,23 +186,27 @@ class PortfoliosGrantsAPIView(SmartPortfolioListMixin,
         .. code-block:: json
 
             {
-               "grantee": "energy-utility"
+               "grantee": {
+                 "slug": "energy-utility",
+                 "full_name": "Energy Utility"
+               }
             }
 
         responds
 
         .. code-block:: json
 
-            {}
+            {
+               "grantee": {
+                 "slug": "energy-utility",
+                 "full_name": "Energy Utility"
+               }
+            }
         """
-        return self.create(request, *args, **kwargs)
+        serializer = self.get_serializer(data=request.data)
+        serializer.is_valid(raise_exception=True)
 
-    def get_serializer_class(self):
-        if self.request.method.lower() == 'post':
-            return PortfolioGrantCreateSerializer
-        return super(PortfoliosGrantsAPIView, self).get_serializer_class()
-
-    def perform_create(self, serializer):
+        #pylint:disable=too-many-locals
         created_at = datetime_or_now()
         # If we don't make a copy, we will get an exception "Got KeyError
         # when attempting to get a value for field `slug`" later on in
@@ -216,27 +231,76 @@ class PortfoliosGrantsAPIView(SmartPortfolioListMixin,
         grantee, unused_created = account_model.objects.get_or_create(
             defaults=grantee_data, **lookups)
 
-        accounts = serializer.validated_data.get('accounts', [])
+        accounts = []
+        # XXX assert self.account has access to each account in `accounts`.
+        # accounts = serializer.validated_data.get('accounts', [])
         if not accounts:
             accounts = [self.account]
-        defaults = {
-            'initiated_by': self.request.user,
-            'state': PortfolioDoubleOptIn.OPTIN_GRANT_INITIATED,
-            'ends_at': created_at
-        }
+
+        # Attempting to grant access to itself is an error.
+        for account in accounts:
+            if account == grantee:
+                raise ValidationError({'grantee': _("The profile already has"\
+                    " access to the data you are trying to share.")})
+
+        status_code = status.HTTP_200_OK
+        ends_at = created_at + relativedelta(months=4)
+        portfolios = []
+        requests_accepted = []
         with transaction.atomic():
             for account in accounts:
-                # XXX assert self.account has access to `account`
-                portfolio, unused_created = \
-                    PortfolioDoubleOptIn.objects.exclude(
-                        state=PortfolioDoubleOptIn.OPTIN_REQUEST_DENIED
-                    ).update_or_create(
-                        account=account,
-                        grantee=grantee,
-                        campaign=campaign,
-                        defaults=defaults)
-                signals.portfolio_grant_initiated.send(sender=__name__,
-                    portfolio=portfolio, request=self.request)
+                # If we have a pending grant initiated, we actualize it.
+                portfolio = PortfolioDoubleOptIn.objects.filter(
+                    Q(account=account) &
+                    Q(grantee=grantee) &
+                    Q(campaign=campaign) &
+                    Q(state=PortfolioDoubleOptIn.OPTIN_GRANT_INITIATED)
+                ).first()
+                if portfolio:
+                    portfolio.initiated_by = self.request.user
+                    portfolio.created_at = created_at
+                    portfolio.ends_at = ends_at
+                    portfolio.save()
+                    portfolios += [portfolio]
+                else:
+                    # If we have a pending request initiated, we grant it.
+                    portfolio = PortfolioDoubleOptIn.objects.filter(
+                        Q(account=account) &
+                        Q(grantee=grantee) &
+                        Q(campaign=campaign) &
+                        Q(state=PortfolioDoubleOptIn.OPTIN_REQUEST_INITIATED)
+                    ).first()
+                    if portfolio:
+                        portfolio.request_accepted()
+                        requests_accepted += [portfolio]
+                    else:
+                        portfolio = PortfolioDoubleOptIn.objects.create(
+                            created_at=created_at,
+                            ends_at=ends_at,
+                            account=account,
+                            grantee=grantee,
+                            campaign=campaign,
+                            initiated_by=self.request.user,
+                            state=PortfolioDoubleOptIn.OPTIN_GRANT_INITIATED,
+                            verification_key=PortfolioDoubleOptIn.generate_key(
+                                account))
+                        status_code = status.HTTP_201_CREATED
+                portfolios += [portfolio]
+
+        for portfolio in requests_accepted:
+            signals.portfolio_request_accepted.send(sender=__name__,
+                portfolio=portfolio, request=self.request)
+        signals.portfolios_grant_initiated.send(sender=__name__,
+            portfolios=portfolios, invitee=grantee_data, request=self.request)
+
+        results = []
+        serializer_class = PortfolioOptInSerializer
+        serializer_kwargs = {'context': self.get_serializer_context()}
+        serializer = serializer_class(**serializer_kwargs)
+        for portfolio in portfolios:
+            results += [serializer.to_representation(portfolio)]
+
+        return HttpResponse(results, status=status_code)
 
 
 class PortfoliosGrantAcceptAPIView(AccountMixin, generics.DestroyAPIView):
@@ -259,14 +323,18 @@ class PortfoliosGrantAcceptAPIView(AccountMixin, generics.DestroyAPIView):
 
         POST /api/energy-utility/portfolios/grants/0123456789abcef HTTP/1.1
 
+    .. code-block:: json
+
+        {}
+
     responds
 
     .. code-block:: json
 
         {}
     """
-    serializer_class = PortfolioOptInSerializer
-    lookup_url_kwarg = 'verification_key'
+    lookup_field = 'verification_key'
+    serializer_class = NoModelSerializer
 
     def get_queryset(self):
         return PortfolioDoubleOptIn.objects.filter(grantee=self.account)
@@ -279,7 +347,7 @@ class PortfoliosGrantAcceptAPIView(AccountMixin, generics.DestroyAPIView):
 
     def delete(self, request, *args, **kwargs):
         """
-        Denies a portfolio grant
+        Denies or removes a portfolio grant
 
         **Tags**: portfolios
 
@@ -290,20 +358,29 @@ class PortfoliosGrantAcceptAPIView(AccountMixin, generics.DestroyAPIView):
             DELETE /api/energy-utility/portfolios/grants/0123456789abcef\
  HTTP/1.1
         """
-        return self.destroy(request, *args, **kwargs)
+        filter_args = {self.lookup_field: self.kwargs.get(self.lookup_field)}
+        try:
+            instance = self.get_queryset().get(**filter_args)
+            instance.state = PortfolioDoubleOptIn.OPTIN_GRANT_DENIED
+            instance.save()
+            signals.portfolio_grant_denied.send(sender=__name__,
+                portfolio=instance, request=self.request)
+            return HttpResponse(status=status.HTTP_204_NO_CONTENT)
+
+        except PortfolioDoubleOptIn.DoesNotExist:
+            pass
+
+        # We cannot find the grant to the grantee, so let's look
+        # if the grant was removed by any chance.
+        instance = generics.get_object_or_404(
+            PortfolioDoubleOptIn.objects.filter(account=self.account),
+            **filter_args)
+        instance.delete()
+        return HttpResponse(status=status.HTTP_204_NO_CONTENT)
 
     def perform_create(self, instance):
-        with transaction.atomic():
-            instance.create_portfolios()
-            instance.state = PortfolioDoubleOptIn.OPTIN_GRANT_ACCEPTED
-            instance.save()
-            signals.portfolio_grant_accepted.send(sender=__name__,
-                portfolio=instance, request=self.request)
-
-    def perform_destroy(self, instance):
-        instance.state = PortfolioDoubleOptIn.OPTIN_GRANT_DENIED
-        instance.save()
-        signals.portfolio_grant_denied.send(sender=__name__,
+        instance.grant_accepted()
+        signals.portfolio_grant_accepted.send(sender=__name__,
             portfolio=instance, request=self.request)
 
 
@@ -360,6 +437,7 @@ class PortfoliosRequestsAPIView(SmartPortfolioListMixin,
             ]
         }
     """
+    lookup_field = settings.ACCOUNT_LOOKUP_FIELD
     serializer_class = PortfolioOptInSerializer
 
     def get_queryset(self):
@@ -382,14 +460,26 @@ class PortfoliosRequestsAPIView(SmartPortfolioListMixin,
         .. code-block:: json
 
             {
-               "account": "supplier-1"
+               "accounts": [
+                 {
+                   "slug": "supplier-1",
+                   "full_name": "Supplier 1"
+                 }
+               ]
             }
 
         responds
 
         .. code-block:: json
 
-            {}
+            {
+               "accounts": [
+                 {
+                   "slug": "supplier-1",
+                   "full_name": "Supplier 1"
+                 }
+               ]
+             }
         """
         return self.create(request, *args, **kwargs)
 
@@ -400,9 +490,10 @@ class PortfoliosRequestsAPIView(SmartPortfolioListMixin,
 
     def perform_create(self, serializer):
         account_model = get_account_model()
+        ends_at = datetime_or_now()
         defaults = {
             'state': PortfolioDoubleOptIn.OPTIN_REQUEST_INITIATED,
-            'ends_at': datetime_or_now(),
+            'ends_at': ends_at,
             'initiated_by': self.request.user
         }
         accounts = serializer.validated_data['accounts']
@@ -414,12 +505,24 @@ class PortfoliosRequestsAPIView(SmartPortfolioListMixin,
                     defaults={
                         'email': account.get('email')
                     }, **lookups)
-                portfolio, unused_created = \
-                    PortfolioDoubleOptIn.objects.update_or_create(
+                campaign = serializer.validated_data.get('campaign')
+                double_optin_queryset = PortfolioDoubleOptIn.objects.filter(
+                    Q(ends_at__isnull=True) | Q(ends_at__gt=ends_at),
+                    grantee=self.account,
+                    account=account,
+                    campaign=campaign).order_by('-created_at')
+                portfolio = double_optin_queryset.first()
+                if portfolio:
+                    portfolio.state = defaults.get('state')
+                    portfolio.ends_at = defaults.get('ends_at')
+                    portfolio.initiated_by = defaults.get('initiated_by')
+                    portfolio.save()
+                else:
+                    portfolio = PortfolioDoubleOptIn.objects.create(
                         grantee=self.account,
                         account=account,
-                        campaign=serializer.validated_data.get('campaign'),
-                        defaults=defaults)
+                        campaign=campaign,
+                        **defaults)
                 requests_initiated += [(portfolio, account)]
 
         for request in requests_initiated:
@@ -447,7 +550,11 @@ class PortfoliosRequestAcceptAPIView(AccountMixin, generics.DestroyAPIView):
 
     .. code-block:: http
 
-        POST /api/energy-utility/portfolios/requests/0123456789abcef HTTP/1.1
+        POST /api/supplier-1/portfolios/requests/0123456789abcef HTTP/1.1
+
+    .. code-block:: json
+
+        {}
 
     responds
 
@@ -455,8 +562,9 @@ class PortfoliosRequestAcceptAPIView(AccountMixin, generics.DestroyAPIView):
 
         {}
     """
-    serializer_class = PortfolioOptInSerializer
-    lookup_url_kwarg = 'verification_key'
+#    lookup_url_kwarg = 'verification_key'
+    lookup_field = 'verification_key'
+    serializer_class = NoModelSerializer
 
     def get_queryset(self):
         return PortfolioDoubleOptIn.objects.filter(account=self.account)
@@ -477,19 +585,15 @@ class PortfoliosRequestAcceptAPIView(AccountMixin, generics.DestroyAPIView):
 
         .. code-block:: http
 
-            DELETE /api/energy-utility/portfolios/requests/0123456789abcef\
+            DELETE /api/supplier-1/portfolios/requests/0123456789abcef\
  HTTP/1.1
         """
         return self.destroy(request, *args, **kwargs)
 
     def perform_create(self, instance):
-        with transaction.atomic():
-            instance.create_portfolios()
-            instance.state = \
-                PortfolioDoubleOptIn.OPTIN_REQUEST_ACCEPTED
-            instance.save()
-            signals.portfolio_request_accepted.send(sender=__name__,
-                portfolio=instance, request=self.request)
+        instance.request_accepted()
+        signals.portfolio_request_accepted.send(sender=__name__,
+            portfolio=instance, request=self.request)
 
     def perform_destroy(self, instance):
         instance.state = PortfolioDoubleOptIn.OPTIN_REQUEST_DENIED
