@@ -1,4 +1,4 @@
-# Copyright (c) 2019, DjaoDjin inc.
+# Copyright (c) 2022, DjaoDjin inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -26,10 +26,10 @@ from django.shortcuts import get_object_or_404
 from django.views.generic import TemplateView, DetailView, ListView
 
 from ..compat import csrf, reverse
-from survey.models import Sample
 from ..mixins import (EditableFilterMixin, MatrixMixin, MatrixQuerysetMixin,
     CampaignMixin)
-from ..models import EditableFilter
+from ..models import EditableFilter, Sample
+from ..utils import update_context_urls
 
 
 class MatrixListView(MatrixQuerysetMixin, ListView):
@@ -58,7 +58,7 @@ class MatrixDetailView(MatrixMixin, DetailView):
         with ``is_selected`` for cohorts the user shows.
         """
         selected = list(self.object.cohorts.all())
-        cohorts = EditableFilter.objects.filter(tags='cohort')
+        cohorts = EditableFilter.objects.filter(extra__contains='cohort')
         for cohort in cohorts:
             if cohort in selected:
                 cohort.is_selected = True
@@ -66,21 +66,21 @@ class MatrixDetailView(MatrixMixin, DetailView):
 
     def get_context_data(self, **kwargs):
         context = super(MatrixDetailView, self).get_context_data(**kwargs)
-        metrics = EditableFilter.objects.filter(tags='metric')
+        metrics = EditableFilter.objects.filter(extra__contains='metric')
         for metric in metrics:
             if metric == self.object.metric:
                 metric.is_selected = True
-        url_kwargs = {'editable_filter': ''}
-        url_kwargs.update(self.get_url_kwargs())
         context.update({
             'cohorts': self.get_cohorts(),
             'metrics': metrics,
-            'editable_filter_api_base': reverse('editable_filter_api',
-                kwargs=url_kwargs),
         })
-        url_kwargs = {self.matrix_url_kwarg: str(self.object)}
-        url_kwargs.update(self.get_url_kwargs())
-        context.update({
+
+        url_kwargs = self.get_url_kwargs()
+        url_filters_kwargs = self.get_url_kwargs()
+        del url_filters_kwargs[self.matrix_url_kwarg]
+        update_context_urls(context, {
+            #'editable_filter_api_base': reverse(
+            #    'survey_api_filter_list', kwargs=url_filters_kwargs),
             'matrix_api': reverse('matrix_api', kwargs=url_kwargs),
         })
         return context
